
if not BWA then return end
local BWA = BWA

local L = BWA.locale

local genericGet = function(info)
	return BWA.db.profile[info[#info]]
end
local genericSet = function(info, value)
	BWA.db.profile[info[#info]] = value
end


local options
local function GetOptions()
	local o = options or {
		type = "group",
		name = "Broker_WrapA",
		childGroups = "tab",
		args = {
			ordering = {
				order = 2,
				name = L["Ordering"],
				type = "group",
				childGroups = "tree",
				args = {},
			},
		},
	}
	options = o
	return options
end
BWA.GetOptions = GetOptions

local function genericObjectGet(info)
	return BWA.db.profile.objectDBs[info.arg][info[#info]]
end
local function genericObjectSet(info, value)
	local max = #info
	BWA.db.profile.objectDBs[info.arg][info[#info]] = value
end
local function setObjectDisplay(info, value)
	genericObjectSet(info, value)
	if info.arg == BWA.activeName then BWA:UpdateObject() end
end
local function setObjectEnable(info, value)
	genericObjectSet(info, value)
	if not value and BWA.activeName == info.arg then BWA:CycleRight() end
	if value and not BWA:GetObjectOrder(info.arg) then BWA:OrderObjectAdd(info.arg) BWA:ReorderOrderOptions() end
end

local objectOptions = {}
local function GetObjectOptions(name)
	local o = objectOptions[name] or {
		type = "group",
		name = name,
		get = genericObjectGet,
		set = genericObjectSet,
		args = {
			enabled = {
				order = 1,
				name = L["Enable"],
				type = "toggle",
				arg = name,
				set = setObjectEnable,
			},
			showText = {
				order = 2,
				name = L["Show text"],
				type = "toggle",
				arg = name,
				set = setObjectDisplay
			},
			showLabel = {
				order = 3,
				name = L["Show label"],
				type = "toggle",
				arg = name,
				set = setObjectDisplay
			},
			showIcon = {
				order = 4,
				name = L["Show icon"],
				type = "toggle",
				arg = name,
				set = setObjectDisplay
			},
			autoshow = {
				order = 6,
				name = L["Show on update"],
				desc = L["Show on update desc"],
				type = "toggle",
				arg = name,
			},
		},
	}
	objectOptions[name] = o
	return o
end		

local function objectOrderUp(info)
	BWA:OrderObjectUp(info.arg)
	BWA:ReorderOrderOptions()
end

local function objectOrderDown(info)
	BWA:OrderObjectDown(info.arg)
	BWA:ReorderOrderOptions()
end

local function objectOrderRemove(info)
	BWA:OrderObjectRemove(info.arg)
	BWA.db.profile.objectDBs[info.arg].enabled = false
	--ReorderOrderOptions()
	local key = gsub(info.arg, " ", "_")
	options.args.ordering.args[key] = nil
end

local function objectEnabledText(info)
	return BWA.objects[info.arg] and (BWA.db.profile.objectDBs[info.arg].enabled and L["Plugin enabled"] or L["Plugin disabled"]) or L["Plugin not loaded"]
end

local cache = {}
local function getObjectOrderingOptions(name, order)
	local t = cache[name] or {
		name = name,
		type = "group",
		args = {
			enabled = {
				order = 1,
				name = objectEnabledText,
				type = "description",
				arg = name,
			},
			up = {
				order = 2,
				name = L["Move up"],
				arg = name,
				type = "execute",
				func = objectOrderUp,
			},
			down = {
				order = 3,
				name = L["Move down"],
				arg = name,
				type = "execute",
				func = objectOrderDown,
			},
			remove = {
				order = 4,
				name = L["Remove from list"],
				desc = L["Remove from list desc"],
				arg = name,
				type = "execute",
				func = objectOrderRemove,
			},
		},
	}
	t.order = order
	cache[name] = t
	return t
end

local temp = {}
function BWA:ReorderOrderOptions()
	local key
	local ordering = options.args.ordering.args
	for k in pairs(ordering) do temp[k] = true end
	for name, order in BWA:IterateOrderedObjects() do
		key = gsub(name, " ", "_")
		temp[key] = nil
		if not ordering[key] then
			ordering[key] = getObjectOrderingOptions(name, order)
		elseif ordering[key].order ~= order then
			ordering[key].order = order
		end
	end
	for k in pairs(temp) do
		ordering[k] = nil
		temp[k] = nil
	end
end

local ChatCmd
do
	local AceCfgCmd = LibStub("AceConfigCmd-3.0")

	function ChatCmd(input)
		if not input or input:trim() == "" then
			InterfaceOptionsFrame_OpenToCategory("Broker_Wraps")
		else
			AceCfgCmd.HandleCommand(self, "Broker_Wraps", "BWA", input)
		end
	end
end

function BWA:RegisterOptions()
	BWA.RegisterOptions = nil
	
	options = GetOptions()
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(BWA.db)
	options.args.profiles.order = 3
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Broker_Wraps", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Broker_Wraps", "Broker_Wraps")

	BWA:RegisterChatCommand("Broker_Wraps", ChatCmd)
	BWA:RegisterChatCommand("wraps", ChatCmd)
end

function BWA:RegisterObjectOptions(name)
	local ooptions = GetObjectOptions(name)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Broker_Wraps"..name, ooptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Broker_Wraps"..name, name, "Broker_Wraps")
	local key = gsub(name, " ", "_")
	if BWA:GetObjectOrder(name) and BWA.db.profile.objectDBs[name].enabled and not options.args.ordering.args[key] then
		options.args.ordering.args[key] = getObjectOrderingOptions(name, BWA:GetObjectOrder(name))
	end
end
